<?php
/*
Parser de archivos .RDF generados por Hera

Hera es una herramienta de revisin de la accesibilidad de pginas web que permite generar informes de cada revisin en formatos XHTML, PDF y RDF.

Este parser (analizador de sintxis) permite utilizar los informes RDF para mostrar un listado ordenado de las revisiones y consultar en detalle los resultados de cada pgina.

Para saber cmo instalar este sistema en su servidor, por favor, consulte el archivo LEEME.txt.

Hera <http://www.sidar.org/hera/>
Autor: Carlos Benavidez <carlos@sidar.org>
*/

class Hera {
	var $rdf = '';
	var $Title = '';
	var $Page = '';
	var $Date = '';
	var $Tester = '';
	var $Email = '';
	var $Comment = '';
	var $Conformance = '';
	var $manual = array();
	var $auto = array();
	var $totales = array(
		'AnotTested' => 0, 'AAnotTested' => 0, 'AAAnotTested' => 0,
		'Afail' => 0, 'AAfail' => 0, 'AAAfail' => 0,
		'Aparcial' => 0, 'AAparcial' => 0, 'AAAparcial' => 0,
		'AcannotTell' => 0, 'AAcannotTell' => 0, 'AAAcannotTell' => 0,
		'Apass' => 0, 'AApass' => 0, 'AAApass' => 0,
		'AnotApplicable' => 0, 'AAnotApplicable' => 0, 'AAAnotApplicable' => 0
		);
	var $puntos = array();
	var $t_duda = 0;
	var $t_mal = 0;
	var $t_parcial = 0;
	var $t_nose = 0;
	var $revisados = array();
	var $lst_A = array("1.1", "1.2", "1.3", "1.4", "2.1", "4.1", "5.1", "5.2", "6.1", "6.2", "6.3", "7.1", "8.1", "9.1", "11.4", "12.1", "14.1");
	var $lst_AA = array("2.2", "3.1", "3.2", "3.3", "3.4", "3.5", "3.6", "3.7", "5.3", "5.4", "6.4", "6.5", "7.2", "7.3", "7.4", "7.5", "9.2", "9.3", "10.1", "10.2", "11.1", "11.2", "12.2", "12.3", "12.4", "13.1", "13.2", "13.3", "13.4");
	var $lst_AAA = array("1.5", "4.2", "4.3", "5.5", "5.6", "9.4", "9.5", "10.3", "10.4", "10.5", "11.3", "13.5", "13.6", "13.7", "13.8", "13.9", "13.10", "14.2", "14.3");
	var $letras = array('A' => 'lst_A', 'AA' => 'lst_AA', 'AAA' => 'lst_AAA');
	var $wcag1 = array(11 => '1.1', 12 => '1.2', 13 => '1.3', 14 => '1.4', 15 => '1.5', 21 => '2.1', 22 => '2.2', 31 => '3.1', 32 => '3.2', 33 => '3.3', 34 => '3.4', 35 => '3.5', 36 => '3.6', 37 => '3.7', 41 => '4.1', 42 => '4.2', 43 => '4.3', 51 => '5.1', 52 => '5.2', 53 => '5.3', 54 => '5.4', 55 => '5.5', 56 => '5.6', 61 => '6.1', 62 => '6.2', 63 => '6.3', 64 => '6.4', 65 => '6.5', 71 => '7.1', 72 => '7.2', 73 => '7.3', 74 => '7.4', 75 => '7.5', 81 => '8.1', 91 => '9.1', 92 => '9.2', 93 => '9.3', 94 => '9.4', 95 => '9.5', 101 => '10.1', 102 => '10.2', 103 => '10.3', 104 => '10.4', 105 => '10.5', 111 => '11.1', 112 => '11.2', 113 => '11.3', 114 => '11.4', 121 => '12.1',  122 => '12.2', 123 => '12.3', 124 => '12.4', 131 => '13.1', 132 => '13.2', 133 => '13.3', 134 => '13.4', 135 => '13.5', 136 => '13.6', 137 => '13.7', 138 => '13.8', 139 => '13.9', 1310 => '13.10', 141 => '14.1', 142 => '14.2', 143 => '14.3');

	function Hera($page) {
		$page = "rdf/".$page;
		if ($temp = @file_get_contents($page)) {
			$this->rdf = $temp;
			$this->Datos();
			$this->Resultados();
		} else {
			$this = NULL;
		}
	}

	// Datos del informe
	function Datos() {
		global $lang;
		if (preg_match("@<rdfs:label[^>]*>(.*)</rdfs:label>@", $this->rdf, $tit)) {
			$this->Title = utf8_decode($tit[1]);
		} else {
			$this->Title = $lang['no_title'];
		}
		if (preg_match("@<earl:reprOf>(.*)</earl:reprOf>@", $this->rdf, $pag)) {
			$this->Page = $pag[1];
		} else {
			$this->Page = $lang['no_page'];
		}
		if (preg_match("@<dc:date[^>]*>(.*)</dc:date>@", $this->rdf, $fec)) {
			$this->Date = date($lang['date_format'], strtotime($fec[1]));
		} else {
			$this->Date = $lang['no_date'];
		}
		if (preg_match("@<foaf:name>(.*)</foaf:name>@", $this->rdf, $aut)) {
			$this->Tester = utf8_decode($aut[1]);
		} else {
			$this->Tester = $lang['no_tester'];
		}
		preg_match("@<foaf:mbox rdf:resource=\"mailto:([^\"]+)@", $this->rdf, $mail);
		$this->Email =  $mail[1];
		preg_match("@<rdfs:comment[^>]*>(.*)</rdfs:comment>@smU", $this->rdf, $coment);
		$this->Comment =  utf8_decode($coment[1]);
	}

	// Analiza el archivo RDF y obtiene los resultados
	function Resultados() {
		global $wcag, $lang;
		// Genera un array con los bloques de resultados
		preg_match_all ("@<earl:Assertion rdf:about=\"#wcag1cp([0-9]{1,2}\.[0-9]{1,2})\">(.*)</earl:Assertion>@smU", $this->rdf, $asser);
		for ($i=0; $i < count($asser[0]); $i++) {
			$pto = $asser[1][$i]; // el punto revisado

				$this->revisados[] = $pto;

			preg_match("@<earl:result (.*)#(pass|fail|parcial|notApplicable|cannotTell|notTested)\"/>@", $asser[2][$i], $re);
			$res = $re[2]; // el resultado del punto

			// Los datos se asignan a distintos arrays,
			// segn sean automticos o manuales
			if (preg_match("@<earl:mode (.*)#automatic\"/>@", $asser[2][$i])) {
				$res = ($res == 'cannotTell')? 'notTested' : $res;
				$this->auto[$pto][0] = $res;
				if (preg_match("@<p [^>]*>(.*)</p>@sm", $asser[2][$i], $com)) {
					$this->auto[$pto][1] = $com[1];
				} else {
					$this->auto[$pto][1] = '';
				}
			} else {
				$this->manual[$pto][0] = $res;
				if (preg_match("@<earl:message>(.*)</earl:message>@", $asser[2][$i], $com)) {
					$this->manual[$pto][1] = $com[1];
				} else {
					$this->manual[$pto][1] = '';
				}
			}
			$pto = '';
			$res = '';
		}

		// Se recorre el array de puntos y se obtiene un nico
		// array $puntos (con el resultado manual, si existe, o,
		// en caso contrario, con el resultado automtico)
		// 0 : prioridad (A, AA, AAA)
		// 1 : manual/auto
		// 2 : resultado
		// 3 : comentario
		foreach ($this->letras as $p => $letra) {
			foreach ($this->$letra as $k => $v) {
				$this->puntos[$v][0] = $p;
			}
		}
		foreach ($this->puntos as $k => $v) {
			if (isset($this->manual[$k][0]) && ($this->manual[$k][0] != '')) {
				$this->puntos[$k][1] = 'manual';
				$this->puntos[$k][2] = $this->manual[$k][0];
				$this->puntos[$k][3] = $this->manual[$k][1];
			} else {
				$this->puntos[$k][1] = 'auto';
				$this->puntos[$k][2] = $this->auto[$k][0];
				$this->puntos[$k][3] = $this->auto[$k][1];
			}
		}
		// Obtiene los totales, por prioridad
		foreach ($this->letras as $p => $arr) {
			foreach ($this->$arr as $k => $v) {
				$x = $p.$this->puntos[$v][2];
				$this->totales[$x]++;
			}
		}

		// Totales de puntos a verificar, incorrectos, parciales y no s
		$this->t_duda = $this->totales['AnotTested'] + $this->totales['AAnotTested'] + $this->totales['AAAnotTested'];
		$this->t_mal = $this->totales['Afail'] + $this->totales['AAfail'] + $this->totales['AAAfail'];
		$this->t_parcial = $this->totales['Aparcial'] + $this->totales['AAparcial'] + $this->totales['AAAparcial'];
		$this->t_nose = $this->totales['AcannotTell'] + $this->totales['AAcannotTell'] + $this->totales['AAAcannotTell'];

		// Define el icono A, AA o AAA
		if ($this->totales['AnotTested'] + $this->totales['AcannotTell'] + $this->totales['Aparcial'] + $this->totales['Afail'] == 0) {
			if ($this->totales['AAnotTested'] + $this->totales['AAcannotTell'] + $this->totales['AAparcial'] + $this->totales['AAfail'] == 0) {
				if ($this->totales['AAAnotTested'] + $this->totales['AAAcannotTell'] + $this->totales['AAAparcial'] + $this->totales['AAAfail'] == 0) {
					$ico_acc = 'AAA';
				} else {
					$ico_acc = 'AA';
				}
			} else {
				$ico_acc = 'A';
			}

			$array_puntos = array_unique($this->revisados);

			if (count($array_puntos) < 65) {
				$faltan = 65 - count($array_puntos);
				$this->Conformance = sprintf($lang['partial_report'], $faltan);
			} else {
				$this->Conformance = ' <img src="img/her_'.$ico_acc.'.gif" alt="'.sprintf($lang['conformance'], $ico_acc).'" class="icoacc" />';
			}
		} else {
			if ($this->totales['AnotTested'] > 0) {
				$this->Conformance = $lang['not_tested_yet'];
			} else {
				$this->Conformance = $lang['not_accessible'];
			}
		}
	}

	// Escribe la tabla resumen
	function Summary() {
		global $lang;
		$res = '';
		$res .= '<table summary="'.$lang['table_summary'].'" cellspacing="0" cellpadding="0" class="totales">'."\n";
		$res .= '<caption>'.$lang['table_caption'].'</caption>'."\n";
		$res .= "<tr>\n";
		$res .= '<th style="width:90px" scope="col">'.$lang['th_priority'].'</th>'."\n";
		$res .= '<th scope="col">'.$lang['th_need_checking'].'</th>'."\n";
		$res .= '<th scope="col">'.$lang['th_pass'].'</th>'."\n";
		$res .= '<th scope="col">'.$lang['th_fail'].'</th>'."\n";
		$res .= '<th scope="col">'.$lang['th_na'].'</th>'."\n";
		if ($this->t_parcial > 0) {
			$res .= '<th scope="col">'.$lang['th_partial'].'</th>'."\n";
		}
		if ($this->t_nose > 0) {
			$res .= '<th scope="col">'.$lang['th_cannot_tell'].'</th>'."\n";
		}
		$res .= "</tr>\n";

		$cel = array ( 2 => 'notTested', 1 => 'pass', 3 => 'fail', 4 => 'notApplicable', 5 => 'parcial', 6 => 'cannotTell' );
			$prx = 'A';
			$num = 1;
			for ($x=1; $x < 4; $x++) {
				$res .= '<tr>'."\n".'<td><a href="#'.$prx.'" title="'.sprintf($lang['title_logo_priority'], $num).'"><img src="img/wcag1'.$prx.'.gif" class="wcag" alt="'.sprintf($lang['alt_logo_priority'], $num).'" /></a></td>'."\n";
				foreach ($cel as $key => $val) {
					if (($key < 5) || (($key == 5) && ($this->t_parcial > 0)) || (($key == 6) && ($this->t_nose > 0))) {
						$res .= '<td class="'.$val.'">'.$this->ico_res($prx, $val).'</td>'."\n";
					}
				}
				$res .= "</tr>\n";
				$prx = $prx.'A';
				$num++;
			}
			$res .= "</table>\n\n";
			return $res;
	}

	function Details() {
		global $wcag, $lang;
		$res = array ( 2 => 'notTested', 1 => 'pass', 3 => 'fail', 4 => 'notApplicable', 5 => 'parcial', 6 => 'cannotTell' );
		$temp = '';
		$priori = 'A';
		$num = 1;
		$det = '';
		for ($x=1; $x < 4; $x++) {
			$det .= '<h2><a name="'.$priori.'" id="'.$priori.'"></a>'.sprintf($lang['h2_priority'], $num).'</h2>'."\n";
			foreach ($res as $key => $resulta) {
				foreach ($this->wcag1 as $k => $v) {
					if (($this->puntos[$v][0] == $priori) && ($this->puntos[$v][2] == $resulta)) {
						$temp .= '<dt>'.sprintf($lang['checkpoint'], $v);
						if ($this->puntos[$v][1] == 'manual') {
							$temp .= ' <img src="img/hera_man.gif" alt="'.$lang['manual_checking'].'" class="ico" />';
						} else {
							$temp .= ' <img src="img/hera_auto.gif" alt="'.$lang['auto_checking'].'" class="ico" />';
						}
						$temp .= "</dt>\n".'<dd class="q"><blockquote title="'.sprintf($lang['checkpoint_text'], $v).'">'.$wcag[$k]."</blockquote></dd>\n";
						$temp .= '<dd class="comentario">'.$lang['comment'].' '.utf8_decode($this->puntos[$v][3])."</dd>\n";
					}
				}
				if ($temp != '') {
					$det .= '<h3 class="'.$priori.$resulta.'"><a name="'.$priori.$resulta.'" id="'.$priori.$resulta.'"></a>'.sprintf($lang['h3_checkpoints'], $num, $this->Traduce($resulta)).'</h3>'."\n";
					$det .= '<dl class="'.$resulta.'">'.$temp."</dl>\n\n";
					$det .= '<p class="volver"><a href="#arriba" title="'.$lang['title_back'].'"><img src="img/top.gif" alt="'.$lang['alt_back'].'" width="20" height="20" /></a></p>'."\n\n";
					$temp = '';
				}
			}
			$priori .= 'A';
			$num++;
		}
		return $det;
	}

	function Traduce($esto) {
		global $lang;
		switch ($esto) {
			case 'pass': return $lang['pass']; break;
			case 'fail': return $lang['fail']; break;
			case 'parcial': return $lang['parcial']; break;
			case 'notApplicable': return $lang['notApplicable']; break;
			case 'cannotTell': return $lang['cannotTell']; break;
			case 'notTested': return $lang['notTested']; break;
		}
	}

	// Crea los iconos de acuerdo al resultado
	function ico_res($priori, $resulta) {
		global $lang;
		if ($priori == 'A') {
			$pri = 1;
		} else if ($priori == 'AA') {
			$pri = 2;
		} else if ($priori == 'AAA') {
			$pri = 3;
		}

		$lang_x = 'see_'.$resulta;
		$txt_tit = sprintf($lang[$lang_x], $pri);
		$res_array = $priori.$resulta;

		if ($this->totales[$res_array] > 0) {
			return '<a href="#'.$res_array.'" title="'.$txt_tit.'">'.$this->totales[$res_array].' <img src="img/'.$resulta.'.gif" alt="'.$lang[$resulta].'" class="iconito" /></a>';
		} else {
			return '--';
		}
	} // Fin function ico_res

} // Fin class Hera

function List_Files($order='url') {
	global $lang;
	$dirpath = "rdf/";
	if ($handle = opendir($dirpath)) {
		$x = 0;
		$Listado = '';
		while (false !== ($archivo = readdir($handle))) {
			if (substr($archivo, -4) == ".rdf") {
				if ($temp = file_get_contents($dirpath.$archivo)) {
					$file[$x] = $archivo;
					if (preg_match("@<rdfs:label[^>]*>(.*)</rdfs:label>@", $temp, $tit)) {
						$title[$x] = utf8_decode($tit[1]);
					} else {
						$title[$x] = $lang['no_title'];
					}
					preg_match("@<earl:reprOf>(.*)</earl:reprOf>@", $temp, $pag);
					$url[$x] = $pag[1];
					$x++;
				}
			}
		}
		closedir($handle);
		if (is_array($$order)) {
			asort($$order);
			foreach ($$order as $k => $v) {
				$Listado .= '<li><strong><a href="'.$_SERVER['PHP_SELF'].'?rdf='.$file[$k].'" title="'.sprintf($lang['list_title'], $title[$k]).'">'.$title[$k].'</a></strong><br />'.$url[$k].'</li>'."\n";
			}
			return "<ul>\n".$Listado."</ul>\n";
		}
	} else {
		return $lang['list_no_dir'];
	}
}
?>